function [I,sigma,phi,N,Z,V,time]=te_wave_MFIE_square(r,N,f,solver)

%   2-D TE plane wave scattering from a PEC square cylinder in free space, using
%   the MFIE formulation (pulse basis, delta testing)
%   I is vector of coefficients for the basis functions
%   sigma is vector of bistatic echo widths at angles phi
%   r is half the length of one side
%   N is the number of nodes for half of a side

% call gaussian quadrature nodes and weights
[pts,wghts]=gaussian_quad(10);

% set EM constants
c=299792458;    lambda=c/f;     omega=2*pi*f;   mu=4*pi*1e-7;
ep=1/(mu*c*c);  eta=sqrt(mu/ep);    k=omega*sqrt(mu*ep);    coeff=k/(4*j);

% set cylinder geometry and vectors
x(1:N,1)=r*ones(N,1);
y(1:N,1)=linspace(0,r,N)';
x(N:2*N-1)=linspace(r,0,N)';
y(N:2*N-1)=r*ones(N,1);
x(2*N-1:3*N-2)=linspace(0,-r,N)';
y(2*N-1:3*N-2)=r*ones(N,1);
x(3*N-2:4*N-3)=-r*ones(N,1);
y(3*N-2:4*N-3)=linspace(r,0,N)';
x(4*N-3:5*N-4)=-r*ones(N,1)';
y(4*N-3:5*N-4)=linspace(0,-r,N)';
x(5*N-4:6*N-5)=linspace(-r,0,N)';
y(5*N-4:6*N-5)=-r*ones(N,1);
x(6*N-5:7*N-6)=linspace(0,r,N)';
y(6*N-5:7*N-6)=-r*ones(N,1);
x(7*N-6:8*N-7)=r*ones(N,1);
y(7*N-6:8*N-7)=linspace(-r,0,N)';
L=sqrt((x(2)-x(1))^2+(y(2)-y(1))^2);
% if (L>lambda/10)
%     error('Cell size is too large.  Refine the mesh!')
% end
t(:,1)=x(2:end)-x(1:end-1);
t(:,2)=y(2:end)-y(1:end-1);
t=t/L;
N=length(x);

% fill Z matrix
Z=zeros(N-1,N-1);
wghts1=wghts*L;
for m=1:N-1
    for n=1:N-1
        if (m==n)
            Z(m,n)=-1/2;
        else
            xm=(x(m+1)+x(m))/2;
            ym=(y(m+1)+y(m))/2;
            pts_xn=pts*(x(n+1)-x(n))+x(n);
            pts_yn=pts*(y(n+1)-y(n))+y(n);
            Rm=sqrt((xm-pts_xn).^2+(ym-pts_yn).^2);
            cost=t(n,1);
            sint=t(n,2);
            Z(m,n)=coeff*sum(wghts1.*(sint*(xm-pts_xn)-cost*(ym-pts_yn)).*besselh(1,2,k*Rm)./Rm);
        end
    end
end

% fill right hand side
V=zeros(N-1,1);
phi=zeros(N-1,1);
for i=1:N-1
    xm=(x(i+1)+x(i))/2;
    ym=(y(i+1)+y(i))/2;
    phi(i,1)=atan2(ym,xm);
    if (phi(i)<0)
        phi(i)=phi(i)+2*pi;
    end
    V(i)=exp(-j*k*xm);
end

% solve linear system
if (solver=='direct')
    tic
    % direct solver
    I=Z\V;
    time=toc;
end
if (solver=='iterat')
    tic
    [L1,U1]=luinc(sparse(Z),1e-5);
    I=qmr(sparse(Z),V,1e-14,length(V),L1,U1);
    time=toc;
end

% plot current
% plot(phi,abs(I),'-o')
% title('Current induced on surface of PEC cylinder')
% xlabel('\phi')
% ylabel('J magnitude')
% xlim([0 2*pi])
% grid on

% compute bistatic echo width
sigma=zeros(N-1,1);
for m=1:N-1
    for n=1:N-1
        omegan=atan2(t(n,2),t(n,1));
        if (omegan<0)
            omegan=omegan+2*pi;
        end
        xn=(x(n+1)+x(n))/2;
        yn=(y(n+1)+y(n))/2;
        sigma(m,1)=sigma(m,1)+I(n)*L*sin(omegan-phi(m))*exp(j*k*(xn*cos(phi(m))+yn*sin(phi(m))));
    end
    sigma(m,1)=(k/4)*abs(sigma(m,1))^2;
end

% plot bistatic echo width
% figure
% plot(phi,sigma,'-o')
% title('Bistatic Echo Width of PEC Cylinder')
% xlabel('\phi')
% ylabel('\sigma_{TE}')
% xlim([0 2*pi])
% grid on