function [I,sigma,phi,time,iter]=tm_wave_EFIE(r,N,f,solver)

%   2-D TM plane wave scattering from a PEC cylinder in free space, using
%   the MFIE formulation (pulse basis, delta testing)
%   r is the radius of the cylinder
%   N-1 is the number of nodes defining the cylinder (N-1 segments)
%   I is vector of coefficients for the basis functions
%   sigma is vector of bistatic echo widths at angles phi

% call gaussian quadrature nodes and weights
[pts,wghts]=gaussian_quad(10);

% set EM constants
c=299792458;    lambda=c/f;     omega=2*pi*f;   mu=4*pi*1e-7;
ep=1/(mu*c*c);  eta=sqrt(mu/ep);    k=omega*sqrt(mu*ep);    coeff=k*eta/4;
gamma=1.78107241799;

% set cylinder geometry and vectors
phi=linspace(0,2*pi,N)';
x=r*cos(phi);   y=r*sin(phi);
L=sqrt((x(2)-x(1))^2+(y(2)-y(1))^2);
% if (L>lambda/10)
%     error('Cell size is too large.  Refine the mesh!')
% end

% fill Z matrix
Zrow=zeros(1,N-1);
wghts1=wghts*L;
for m=1
    for n=1:N-1
        if (m==n)
            % approximation for self term
            Zrow(n)=(k*eta*L/4)*(1-j*2*(log(gamma*k*L/4)-1)/pi);
        else
            % numerical quadrature of integral
            xm=(x(m+1)+x(m))/2;
            ym=(y(m+1)+y(m))/2;
            pts_xn=pts*(x(n+1)-x(n))+x(n);
            pts_yn=pts*(y(n+1)-y(n))+y(n);
            Rm=sqrt((xm-pts_xn).^2+(ym-pts_yn).^2);
            Zrow(n)=coeff*sum(wghts1.*besselh(0,2,k*Rm));
        end
    end
end

% fill right hand side
V=zeros(N-1,1);
clear phi
for i=1:N-1
    xm=(x(i+1)+x(i))/2;
    ym=(y(i+1)+y(i))/2;
    phi(i,1)=atan2(ym,xm);
    if (phi(i)<0)
        phi(i)=phi(i)+2*pi;
    end
    V(i)=eta*exp(-j*k*xm);
end

% solve linear system
if (solver=='direct')
    Z=toeplitz(transpose(Zrow),Zrow);
    tic
    % direct solver
    I=Z\V;
    time=toc;
    iter=0;
end
if (solver=='conjgr')
    tic
    % CG-FFT
    [I,iter]=cgfft(Zrow,V,1e-14);
    time=toc;
end

% plot current
% plot(phi,abs(I),'-o')
% title('Current induced on surface of PEC cylinder')
% xlabel('\phi')
% ylabel('J magnitude')
% xlim([0 2*pi])
% grid on

% compute bistatic echo width
sigma=zeros(N-1,1);
for m=1:N-1
    for n=1:N-1
        xn=(x(n+1)+x(n))/2;
        yn=(y(n+1)+y(n))/2;
        sigma(m,1)=sigma(m,1)+I(n)*L*exp(j*k*(xn*cos(phi(m))+yn*sin(phi(m))));
    end
    sigma(m,1)=(k*eta*eta/4)*abs(sigma(m,1))^2;
end

% plot bistatic echo width
%figure
%plot(phi,sigma,'-o')
%title('Bistatic Echo Width of PEC Cylinder')
%xlabel('\phi')
%ylabel('\sigma_{TE}')
%xlim([0 2*pi])
%grid on