function [I,phi]=te_wave(N)

%   2-D TE plane wave scattering from a PEC cylinder in free space
%   N-1 is the number of nodes defining the cylinder (n-1 segments)
%   I is vector of coefficients for the basis functions
%   phi is vector of angles
%   Note: this code uses an approximation for the Z-matrix entries.

% set EM constants
f=299792458;  c=299792458;    lambda=c/f;     omega=2*pi*f;   mu=4*pi*1e-7;
ep=1/(mu*c*c);  eta=sqrt(mu/ep);    k=omega*sqrt(mu*ep);    coeff=k/(4*j);

% set cylinder geometry and vectors
r=2*lambda;     phi=linspace(0,2*pi,N)';
x=r*cos(phi);   y=r*sin(phi);
L=sqrt((x(2)-x(1))^2+(y(2)-y(1))^2);
if (L>lambda/10)
    error('Cell size is too large.  Refine the mesh!')
end
t(:,1)=x(2:end)-x(1:end-1);
t(:,2)=y(2:end)-y(1:end-1);
t=t/L;
% fill Z matrix
Z=zeros(N-1,N-1);
for m=1:N-1
    for n=1:N-1
        if (m==n)
            Z(m,n)=-1/2;
        else
            xm=(x(m+1)+x(m))/2;
            ym=(y(m+1)+y(m))/2;
            xn=(x(n+1)+x(n))/2;
            yn=(y(n+1)+y(n))/2;
            Rmn=sqrt((xm-xn).^2+(ym-yn).^2);
            cost=t(n,1);
            sint=t(n,2);
            Z(m,n)=coeff*L*(sint*(xm-xn)/Rmn-cost*(ym-yn)/Rmn)*besselh(1,2,k*Rmn);
        end
    end
end
% fill right hand side
V=zeros(N-1,1);
clear phi
for i=1:N-1
    xm=(x(i+1)+x(i))/2;
    ym=(y(i+1)+y(i))/2;
    phi(i,1)=atan2(ym,xm);
    if (phi(i)<0)
        phi(i)=phi(i)+2*pi;
    end
    V(i)=exp(-j*k*xm);
end
% solve linear system
I=Z\V;
% plot results
figure
plot(phi,abs(I),'-o')
title('Current induced on surface of PEC cylinder')
xlabel('\phi')
ylabel('J magnitude')
xlim([0 2*pi])
grid on

