function [I,sigma, phi,Z,V]=te_wave_EFIE(r,N)

%   2-D TE plane wave scattering from a PEC cylinder in free space, using
%   the EFIE formulation (triangle basis, pulse testing)
%   r is the radius of the cylinder
%   N-1 is the number of nodes defining the cylinder (N-1 segments)
%   I is vector of coefficients for the basis functions
%   sigma is vector of bistatic echo widths at angles phi
%   Note: this code uses numerical quadrature to compute the integrals
%         in the Z-matrix entries.
[pts,wghts]=gaussian_quad(10);

% set EM constants
f=299792458;  c=299792458;    lambda=c/f;     omega=2*pi*f;   mu=4*pi*1e-7;
ep=1/(mu*c*c);  eta=sqrt(mu/ep);    k=omega*sqrt(mu*ep);    
coeff1=k*eta/4; coeff2=eta/(4*k);

% set cylinder geometry and vectors
phi=linspace(0,2*pi,N)';
x=r*cos(phi);   y=r*sin(phi);
L=sqrt((x(2)-x(1))^2+(y(2)-y(1))^2);
if (L>lambda/10)
    error('Cell size is too large.  Refine the mesh!')
end
t(:,1)=x(2:end)-x(1:end-1);
t(:,2)=y(2:end)-y(1:end-1);
t=[t(end,1) t(end,2)
   t(:,1)   t(:,2)];
x=[x(end-1)
   x(:)     ];
y=[y(end-1)
   y(:)     ];
t=t/L;

% fill Z matrix
Z=zeros(N-1,N-1);
wghts1=wghts*L;
wghtshalf=wghts*L/2;
for m=1
    for n=1:N-1
        xnplus1=x(n+2); xn=x(n+1); xnminus1=x(n);
        xnplushalf=(xnplus1+xn)/2; xnminushalf=(xn+xnminus1)/2;
        xm=x(m+1);
        xmplushalf=(x(m+2)+x(m+1))/2;
        xmminushalf=(x(m+1)+x(m))/2;
            pts_xn1=pts*(xn-xnminus1)+xnminus1;
            pts_xn2=pts*(xnplus1-xn)+xn;
            pts_xn3=pts*(xn-xnminushalf)+xnminushalf;
            pts_xn4=pts*(xnplushalf-xn)+xn;
        ynplus1=y(n+2); yn=y(n+1); ynminus1=y(n);
        ynplushalf=(ynplus1+yn)/2; ynminushalf=(yn+ynminus1)/2;
        ym=y(m+1);
        ymplushalf=(y(m+2)+y(m+1))/2;
        ymminushalf=(y(m+1)+y(m))/2;
            pts_yn1=pts*(yn-ynminus1)+ynminus1;
            pts_yn2=pts*(ynplus1-yn)+yn;
            pts_yn3=pts*(yn-ynminushalf)+ynminushalf;
            pts_yn4=pts*(ynplushalf-yn)+yn;
        R1_int1=sqrt((xmminushalf-pts_xn1).^2+(ymminushalf-pts_yn1).^2);
        R1_int2=sqrt((xmminushalf-pts_xn2).^2+(ymminushalf-pts_yn2).^2);
        R2_int1=sqrt((xmplushalf-pts_xn1).^2+(ymplushalf-pts_yn1).^2);
        R2_int2=sqrt((xmplushalf-pts_xn2).^2+(ymplushalf-pts_yn2).^2);
        R_hat1=sqrt((xm-pts_xn3).^2+(ym-pts_yn3).^2);
        R_hat2=sqrt((xm-pts_xn4).^2+(ym-pts_yn4).^2);
        integral1=sum(wghts1.*besselh(0,2,k*R2_int1));
        integral2=sum(wghts1.*besselh(0,2,k*R2_int2));
        integral3=sum(wghts1.*besselh(0,2,k*R1_int1));
        integral4=sum(wghts1.*besselh(0,2,k*R1_int2));
        integral5=sum(wghtshalf.*besselh(0,2,k*R_hat1));
        integral6=sum(wghtshalf.*besselh(0,2,k*R_hat2));
        Z1=coeff1*((L*t(m,:)/2+L*t(m+1,:)/2)*transpose(t(n,:)*integral5+t(n+1,:)*integral6));
        Z2=(coeff2/L)*(integral1-integral2-integral3+integral4);
        Zrow(n)=Z1+Z2;
    end
end
Z=toeplitz(transpose(Zrow),Zrow);

% fill right hand side
V=zeros(N-1,1);
for m=1:N-1
    xm=x(m+1);
    xmplushalf=(x(m+2)+x(m+1))/2;
    xmminushalf=(x(m+1)+x(m))/2;
    pts_xm1=pts*(xm-xmminushalf)+xmminushalf;
    pts_xm2=pts*(xmplushalf-xm)+xm;
    integral1=sum(wghtshalf.*(t(m,2)*eta*exp(-j*k*pts_xm1)));
    integral2=sum(wghtshalf.*(t(m+1,2)*eta*exp(-j*k*pts_xm2)));
    V(m)=integral1+integral2;
end

% solve linear system
I=Z\V;

% plot current
%myPlot = figure(1)
%set(myPlot,'Position',[200,200,400,330])
% figure
% plot(phi(1:end-1),abs(I),'-o')
% title('Current induced on surface of PEC cylinder')
% xlabel('\phi')
% ylabel('J magnitude')
% xlim([0 2*pi])
% grid on

%compute bistatic echo width
sigma=zeros(N-1,1);
for m=1:N-1
   for n=1:N-1
       omegan=atan2(t(n,2),t(n,1));
       if (omegan<0)
           omegan=omegan+2*pi;
       end
       xn=(x(n+1)+x(n))/2;
       yn=(y(n+1)+y(n))/2;
       sigma(m,1)=sigma(m,1)+I(n)*L*sin(omegan-phi(m))*exp(j*k*(xn*cos(phi(m))+yn*sin(phi(m))));
   end
   sigma(m,1)=(k/4)*abs(sigma(m,1))^2;
end

% plot bistatic echo width
%figure
%plot(phi,sigma)
%title('Bistatic Echo Width of PEC Cylinder')
%xlabel('\phi')
%ylabel('\sigma_{TE}')
%xlim([0 2*pi])
%grid on