function [I,sigma,phi]=tm_wave_EFIE(r,N)
solver = 'direct'
%   2-D TM plane wave scattering from a PEC cylinder in free space
%   r is the radius of the cylinder
%   N is the number of nodes defining the cylinder (N-1 segments)
%   I is vector of coefficients for the basis functions
%   sigma is vector of bistatic echo widths at angles phi
%   Note: this code uses numerical quadrature to compute the integrals
%         in the Z-matrix entries.

[pts,wghts]=gaussian_quad(10);

% set EM constants
f=299792458;  c=299792458;    lambda=c/f;     omega=2*pi*f;   mu=4*pi*1e-7;
ep=1/(mu*c*c);  k=omega*sqrt(mu*ep);    coeff=omega*mu/4.0;

% set cylinder geometry and vectors
phi=linspace(0,2*pi,N)';
x=r*cos(phi);   y=r*sin(phi);
L=sqrt((x(2)-x(1))^2+(y(2)-y(1))^2);
if (L>lambda/10)
    error('Cell size is too large.  Refine the mesh!')
end

t(:,1)=x(2:end)-x(1:end-1);
t(:,2)=y(2:end)-y(1:end-1);
t=[t(end,1) t(end,2)
   t(:,1)   t(:,2)];
x=[x(end-1)
   x(:)     ];
y=[y(end-1)
   y(:)     ];
t=t/L;
% fill Z matrix
Zrow=zeros(1,N-1);
wghts1=wghts*L;

for m=N-1:-1:1
    for n=N-1
        xm=(x(m+1)+x(m))/2;
        ym=(y(m+1)+y(m))/2;
        if (x(n)>x(n+1))
            pts_xn=pts*(x(n)-x(n+1))+x(n+1);
        else
            pts_xn=pts*(x(n+1)-x(n))+x(n);
        end
        if (y(n)>y(n+1))
            pts_yn=pts*(y(n)-y(n+1))+y(n+1);
        else
            pts_yn=pts*(y(n+1)-y(n))+y(n);
        end
        Rm=sqrt((xm-pts_xn).^2+(ym-pts_yn).^2);
        Zrow(m)=coeff*sum(wghts1.*besselh(0,2,k*Rm));
    end
end
Z=toeplitz(Zrow,Zrow);

% fill right hand side
V=zeros(N-1,1);
clear phi
for i=1:N-1
    xm=(x(i+1)+x(i))/2;
    ym=(y(i+1)+y(i))/2;
    phi(i,1)=atan2(ym,xm);
    if (phi(i)<0)
        phi(i)=phi(i)+2*pi;
    end
    V(i)=exp(-j*k*xm);
end
size(Z)

% solve linear system
I=Z\V;

% plot current
% myPlot = figure(1);
% set(myPlot,'Position',[200,200,400,330])
% plot(phi,abs(I),'-o')
% title('Current induced on surface of PEC cylinder')
% xlabel('\phi')
% ylabel('J magnitude')
% xlim([0 2*pi])
% grid on

% compute bistatic echo width
sigma=zeros(N-1,1);
for m=1:N-1
    for n=1:N-1
        omegan=atan2(t(n,2),t(n,1));
        if (omegan<0)
            omegan=omegan+2*pi;
        end
        xn=(x(n+1)+x(n))/2;
        yn=(y(n+1)+y(n))/2;
        sigma(m,1)=sigma(m,1)+I(n)*L*sin(omegan-phi(m))*exp(j*k*(xn*cos(phi(m))+yn*sin(phi(m))));
    end
    sigma(m,1)=(k/4)*abs(sigma(m,1))^2;
end

% plot bistatic echo width
%figure
%plot(phi,sigma)
%title('Bistatic Echo Width of PEC Cylinder')
%xlabel('\phi')
%ylabel('\sigma_{TE}')
%xlim([0 2*pi])
%grid on