%function [I,sigma,phi,time]=tm_wave_EFIE_square(r,N)


%   2-D TE plane wave scattering from a PEC square cylinder in free space, using
%   the MFIE formulation (pulse basis, delta testing)
%   I is vector of coefficients for the basis functions
%   sigma is vector of bistatic echo widths at angles phi
%   r is half the length of one side
%   N is the number of nodes for half of a side

r= 1/(2*pi);
N = 30;
%solver = 'direct'


% call gaussian quadrature nodes and weights
[pts,wghts]=gaussian_quad(10);

% set EM constants
f=299792458;  c=299792458;    lambda=c/f;     omega=2*pi*f;   mu=4*pi*1e-7;
ep=1/(mu*c*c);  eta=sqrt(mu/ep);    k=omega*sqrt(mu*ep);    coeff=k*eta/4;
gamma=1.78107241799;

% set cylinder geometry and vectors
x(1:N,1)=r*ones(N,1);
y(1:N,1)=linspace(0,r,N)';
x(N:2*N-1)=linspace(r,0,N)';
y(N:2*N-1)=r*ones(N,1);
x(2*N-1:3*N-2)=linspace(0,-r,N)';
y(2*N-1:3*N-2)=r*ones(N,1);
x(3*N-2:4*N-3)=-r*ones(N,1);
y(3*N-2:4*N-3)=linspace(r,0,N)';
x(4*N-3:5*N-4)=-r*ones(N,1)';
y(4*N-3:5*N-4)=linspace(0,-r,N)';
x(5*N-4:6*N-5)=linspace(-r,0,N)';
y(5*N-4:6*N-5)=-r*ones(N,1);
x(6*N-5:7*N-6)=linspace(0,r,N)';
y(6*N-5:7*N-6)=-r*ones(N,1);
x(7*N-6:8*N-7)=r*ones(N,1);
y(7*N-6:8*N-7)=linspace(-r,0,N)';
%x = [x(6*N-5:8*N-7); x(1:6*N-4)]
%y = [y(6*N-5:8*N-7); y(1:6*N-4)]
L=sqrt((x(2)-x(1))^2+(y(2)-y(1))^2);
if (L>lambda/10)
    error('Cell size is too large.  Refine the mesh!')
end
N=length(x);

% fill Z matrix
Z=zeros(N-1,N-1);
wghts1=wghts*L;
for m=1:N-1
    for n=1:N-1
        if (m==n)
            % approximation for self term
            Z(m,n)=(k*eta*L/4)*(1-j*2*(log(gamma*k*L/4)-1)/pi);
        else
            % numerical quadrature of integral
            xm=(x(m+1)+x(m))/2;
            ym=(y(m+1)+y(m))/2;
            pts_xn=pts*(x(n+1)-x(n))+x(n);
            pts_yn=pts*(y(n+1)-y(n))+y(n);
            Rm=sqrt((xm-pts_xn).^2+(ym-pts_yn).^2);
            Z(m,n)=coeff*sum(wghts1.*besselh(0,2,k*Rm));
        end
    end
end

% fill right hand side
V=zeros(N-1,1);
for i=1:N-1
    xm=(x(i+1)+x(i))/2;
    ym=(y(i+1)+y(i))/2;
    phi(i,1)=atan2(ym,xm);
    if (phi(i)<0)
        phi(i)=phi(i)+2*pi;
    end
    V(i)=eta*exp(-j*k*xm);
end

% solve linear system
% if (solver=='direct')
%     tic
%     % direct solver
    I=Z\V;
%     time=toc;
% end
% if (solver=='conjgr')
%     tic
%     CG-FFT
%     [I,iter]=cgfft(Z,V,1e-16);
%    time=toc;
% end

save TMI I
load umashankarj.csv
myPlot1 = figure(1);
set(myPlot1,'Position',[200,200,400,330])
% plot current
plot(phi/2/pi-0.5,abs(I),'-o', umashankarj(:,1), umashankarj(:,2),'r')
%title('Current induced on surface of PEC cylinder')
xlabel('Normalized Distance Along Cylinder')
ylabel('|J|/|H|')
xlim([0 0.5])
legend('MOM code','reference')
grid on

% compute bistatic echo width
sigma=zeros(N-1,1);
for m=1:N-1
    for n=1:N-1
        xn=(x(n+1)+x(n))/2;
        yn=(y(n+1)+y(n))/2;
        sigma(m,1)=sigma(m,1)+I(n)*L*exp(j*k*(xn*cos(phi(m))+yn*sin(phi(m))));
    end
    sigma(m,1)=(k*eta*eta/4)*abs(sigma(m,1))^2;
end

save TMS sigma
load umashankarrc.csv
myPlot2 = figure(2);
set(myPlot2,'Position',[200,200,400,330])
% plot bistatic echo width
plot(phi*180/pi,sigma/max(sigma),'o',umashankarrc(:,1),umashankarrc(:,2).^2/max(umashankarrc(:,2))^2,'r')
%title('Bistatic Echo Width of PEC Cylinder')
xlabel('\phi [degrees]')
ylabel('Normalized \sigma_{TM}')
legend('MOM code', 'reference')
xlim([0 180])
ylim([0 1])
grid on
